﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Infragistics.UltraChart.Core.Primitives;
using Infragistics.UltraChart.Core;

namespace ChartAndGrid
{
    public partial class Form1 : Form
    {
        // This flag ensures that the chart is scrolled to the correct position
        // after the maximum values are set for the first time
        bool bMaximumSet = false;
        Random random = new Random();
        
        // See below for PointD definition
        PointD maxDataPoint = new PointD();

        public Form1()
        {
            InitializeComponent();
        }

        private void ultraGrid1_AfterRowActivate(object sender, EventArgs e)
        {
            if (!bMaximumSet)
                return;

            ScrollToActiveDataPoint();
        }

        private void ScrollToActiveDataPoint()
        {
            // Values need to be cast to int because that is the column's data type
            PointD dataPoint = new PointD();
            dataPoint.X = (int)(ultraGrid1.ActiveRow.Cells[1].Value);
            dataPoint.Y = (int)(ultraGrid1.ActiveRow.Cells[2].Value);

            // Center the selected data point
            // Do some math to find the actual scrollbar position
            dataPoint.X -= maxDataPoint.X / 2.0;
            dataPoint.X /= 1 - ultraChart1.Axis.X.ScrollScale.Scale;
            dataPoint.X += maxDataPoint.X / 2.0;

            dataPoint.Y -= maxDataPoint.Y / 2.0;
            dataPoint.Y /= 1 - ultraChart1.Axis.Y.ScrollScale.Scale;
            dataPoint.Y += maxDataPoint.Y / 2.0;
            /////////////////////////////////

            // Figure out where the data point is on the chart
            double xPct = dataPoint.X / maxDataPoint.X;
            double yPct = dataPoint.Y / maxDataPoint.Y;

            // Simple bounds check on our percentages
            if (xPct > 1.0)
                xPct = 1.0;
            else if (xPct < 0.0)
                xPct = 0.0;

            if (yPct > 1.0)
                yPct = 1.0;
            else if (yPct < 0.0)
                yPct = 0.0;

            ultraChart1.Axis.X.ScrollScale.Scroll = xPct;
            ultraChart1.Axis.Y.ScrollScale.Scroll = yPct;
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            // Generate some random data
            for (int i = 0; i < 5; i++)
            {
                ultraDataSource1.Rows.Add(new object [] { random.Next(100), random.Next(100), random.Next(100), random.Next(100) });
            }
        }

        private void ultraChart1_FillSceneGraph(object sender, Infragistics.UltraChart.Shared.Events.FillSceneGraphEventArgs e)
        {
            // If the maximum values are changed, this flag will need to be toggled off
            // This is not implemented in this sample, as the axis ranges are not changed here
            if (bMaximumSet)
                return;

            IAdvanceAxis xAxis = (IAdvanceAxis)e.Grid["X"];
            IAdvanceAxis yAxis = (IAdvanceAxis)e.Grid["Y"];

            if (xAxis != null && yAxis != null)
            {
                maxDataPoint.X = (double)xAxis.Maximum;
                maxDataPoint.Y = (double)yAxis.Maximum;

                bMaximumSet = true;
                ScrollToActiveDataPoint();
            }
        }
    }

    // Point class using doubles to avoid a few unnecessary casts
    public class PointD
    {
        public double X { get; set; }
        public double Y { get; set; }

        public PointD() { X = 0.0; Y = 0.0; }
    }
}
